# coding: utf-8

from czsc.enum import Factors, Signals

def test_factors():
    for k, v in Factors.__members__.items():
        if k == "Other":
            continue
        assert k == v.value.split("~")[0], "{} - {}".format(k, v.value)

def test_signals():
    for k, v in Signals.__members__.items():
        if k == "Other":
            continue
        assert k == v.value.split("~")[0], "{} - {}".format(k, v.value)




# coding: utf-8
import os
import webbrowser
from typing import List
from collections import OrderedDict
from datetime import datetime
import pandas as pd
import traceback
from .objects import Mark, Direction, BI, FX, RawBar, NewBar
from .utils.echarts_plot import kline_pro
from .signals import check_three_fd, check_five_fd, check_seven_fd, check_nine_fd, check_eleven_fd, Signals
from .utils.ta import RSQ


def remove_include(k1: NewBar, k2: NewBar, k3: RawBar):
    """去除包含关系：输入三根k线，其中k1和k2为没有包含关系的K线，k3为原始K线"""
    if k1.high < k2.high:
        direction = Direction.Up
    elif k1.high > k2.high:
        direction = Direction.Down
    else:
        k4 = NewBar(symbol=k3.symbol, dt=k3.dt, open=k3.open,
                    close=k3.close, high=k3.high, low=k3.low, vol=k3.vol, elements=[k3])
        return False, k4

    # 判断 k2 和 k3 之间是否存在包含关系，有则处理
    if (k2.high <= k3.high and k2.low >= k3.low) or (k2.high >= k3.high and k2.low <= k3.low):
        if direction == Direction.Up:
            high = max(k2.high, k3.high)
            low = max(k2.low, k3.low)
            dt = k2.dt if k2.high > k3.high else k3.dt
        elif direction == Direction.Down:
            high = min(k2.high, k3.high)
            low = min(k2.low, k3.low)
            dt = k2.dt if k2.low < k3.low else k3.dt
        else:
            raise ValueError

        if k3.open > k3.close:
            open_ = high
            close = low
        else:
            open_ = low
            close = high
        vol = k2.vol + k3.vol
        # 这里有一个隐藏Bug，len(k2.elements) 在一些及其特殊的场景下会有超大的数量，具体问题还没找到；
        # 临时解决方案是直接限定len(k2.elements)<=100
        elements = [x for x in k2.elements[:100] if x.dt != k3.dt] + [k3]
        k4 = NewBar(symbol=k3.symbol, dt=dt, open=open_,
                    close=close, high=high, low=low, vol=vol, elements=elements)
        return True, k4
    else:
        k4 = NewBar(symbol=k3.symbol, dt=k3.dt, open=k3.open,
                    close=k3.close, high=k3.high, low=k3.low, vol=k3.vol, elements=[k3])
        return False, k4


def check_fx(k1: NewBar, k2: NewBar, k3: NewBar):
    """查找分型"""
    fx = None
    if k1.high < k2.high > k3.high:
        power = "强" if k3.close < k1.low else "弱"
        # 根据 k1 与 k2 是否有缺口，选择 low
        low = k1.low if k1.high > k2.low else k2.low

        # 不允许分型的 high == low
        if low == k2.high:
            low = k1.low

        fx = FX(symbol=k1.symbol, dt=k2.dt, mark=Mark.G, high=k2.high, low=low,
                fx=k2.high, elements=[k1, k2, k3], power=power)
    if k1.low > k2.low < k3.low:
        power = "强" if k3.close > k1.high else "弱"
        # 根据 k1 与 k2 是否有缺口，选择 high
        high = k1.high if k1.low < k2.high else k2.high

        # 不允许分型的 high == low
        if high == k2.low:
            high = k1.high

        fx = FX(symbol=k1.symbol, dt=k2.dt, mark=Mark.D, high=high, low=k2.low,
                fx=k2.low, elements=[k1, k2, k3], power=power)
    return fx


def check_bi(bars: List[NewBar]):
    """输入一串无包含关系K线，查找其中的一笔"""
    fxs = []
    for i in range(1, len(bars)-1):
        fx = check_fx(bars[i-1], bars[i], bars[i+1])
        if isinstance(fx, FX):
            fxs.append(fx)

    if len(fxs) < 2:
        return None, bars

    fx_a = fxs[0]
    try:
        if fxs[0].mark == Mark.D:
            direction = Direction.Up
            fxs_b = [x for x in fxs if x.mark == Mark.G and x.dt > fx_a.dt and x.fx > fx_a.fx]
            if not fxs_b:
                return None, bars
            fx_b = fxs_b[0]
            for fx in fxs_b:
                if fx.high >= fx_b.high:
                    fx_b = fx
        elif fxs[0].mark == Mark.G:
            direction = Direction.Down
            fxs_b = [x for x in fxs if x.mark == Mark.D and x.dt > fx_a.dt and x.fx < fx_a.fx]
            if not fxs_b:
                return None, bars
            fx_b = fxs_b[0]
            for fx in fxs_b[1:]:
                if fx.low <= fx_b.low:
                    fx_b = fx
        else:
            raise ValueError
    except:
        traceback.print_exc()
        return None, bars

    bars_a = [x for x in bars if fx_a.elements[0].dt <= x.dt <= fx_b.elements[2].dt]
    bars_b = [x for x in bars if x.dt >= fx_b.elements[0].dt]

    max_b = max([x.high for x in bars_b])
    min_b = min([x.low for x in bars_b])

    ab_include = (fx_a.high > fx_b.high and fx_a.low < fx_b.low) or (fx_a.high < fx_b.high and fx_a.low > fx_b.low)
    power_price = round(abs(fx_b.fx - fx_a.fx), 2)
    change = round((fx_b.fx - fx_a.fx) / fx_a.fx, 4)
    fxs_ = [x for x in fxs if fx_a.dt <= x.dt <= fx_b.dt]

    if len(bars_a) >= 7 and not ab_include and (fx_b.high < max_b or fx_b.low > min_b):
        bi = BI(symbol=fx_a.symbol, fx_a=fx_a, fx_b=fx_b, fxs=fxs_,
                direction=direction, power=power_price, high=max(fx_a.high, fx_b.high),
                low=min(fx_a.low, fx_b.low), bars=bars_a, length=len(bars_a),
                rsq=RSQ([x.close for x in bars_a[1:-1]]), change=change)
        return bi, bars_b
    else:
        return None, bars


def get_sub_span(bis: List[BI], start_dt: [datetime, str], end_dt: [datetime, str], direction: Direction) -> List[BI]:
    """获取子区间（这是进行多级别联立分析的关键步骤）

    :param bis: 笔的列表
    :param start_dt: 子区间开始时间
    :param end_dt: 子区间结束时间
    :param direction: 方向
    :return: 子区间
    """
    start_dt = pd.to_datetime(start_dt)
    end_dt = pd.to_datetime(end_dt)
    sub = []
    for bi in bis:
        if bi.fx_b.dt > start_dt > bi.fx_a.dt:
            sub.append(bi)
        elif start_dt <= bi.fx_a.dt < bi.fx_b.dt <= end_dt:
            sub.append(bi)
        elif bi.fx_a.dt < end_dt < bi.fx_b.dt:
            sub.append(bi)
        else:
            continue

    if len(sub) > 0 and sub[0].direction != direction:
        sub = sub[1:]
    if len(sub) > 0 and sub[-1].direction != direction:
        sub = sub[:-1]
    return sub


def get_sub_bis(bis: List[BI], bi: BI) -> List[BI]:
    """获取大级别笔对象对应的小级别笔走势

    :param bis: 小级别笔列表
    :param bi: 大级别笔对象
    :return:
    """
    sub_bis = get_sub_span(bis, start_dt=bi.fx_a.dt, end_dt=bi.fx_b.dt, direction=bi.direction)
    if not sub_bis:
        return []
    return sub_bis


class CZSC:
    def __init__(self, bars: List[RawBar], freq: str, max_bi_count=30):
        """

        :param bars: K线数据
        :param freq: K线级别
        :param max_bi_count: 最大保存的笔数量
            默认值为 30，仅使用内置的信号和因子，不需要调整这个参数。
            如果进行新的信号计算需要用到更多的笔，可以适当调大这个参数。
        """
        self.max_bi_count = max_bi_count
        self.bars_raw = []  # 原始K线序列
        self.bars_ubi = []  # 未完成笔的无包含K线序列
        self.bi_list: List[BI] = []
        self.symbol = bars[0].symbol
        self.freq = freq

        for bar in bars:
            self.update(bar)
        self.signals = self.get_signals()

    def __repr__(self):
        return "<CZSC for {}>".format(self.symbol)

    def __update_bi(self):
        bars_ubi = self.bars_ubi
        if len(bars_ubi) < 3:
            return

        # 查找笔
        if not self.bi_list:
            # 第一个笔的查找
            fxs = []
            for i in range(1, len(bars_ubi)-1):
                fx = check_fx(bars_ubi[i-1], bars_ubi[i], bars_ubi[i+1])
                if isinstance(fx, FX):
                    fxs.append(fx)
            if not fxs:
                return

            fx_a = fxs[0]
            fxs_a = [x for x in fxs if x.mark == fx_a.mark]
            for fx in fxs_a:
                if (fx_a.mark == Mark.D and fx.low <= fx_a.low) \
                        or (fx_a.mark == Mark.G and fx.high >= fx_a.high):
                    fx_a = fx
            bars_ubi = [x for x in bars_ubi if x.dt >= fx_a.elements[0].dt]

            bi, bars_ubi_ = check_bi(bars_ubi)
            if isinstance(bi, BI):
                self.bi_list.append(bi)
            self.bars_ubi = bars_ubi_
            return

        last_bi = self.bi_list[-1]

        # 如果上一笔被破坏，将上一笔的bars与bars_ubi进行合并
        min_low_ubi = min([x.low for x in bars_ubi[2:]])
        max_high_ubi = max([x.high for x in bars_ubi[2:]])

        if last_bi.direction == Direction.Up and max_high_ubi > last_bi.high:
            if min_low_ubi < last_bi.low and len(self.bi_list) > 2:
                bars_ubi_a = self.bi_list[-2].bars \
                             + [x for x in self.bi_list[-1].bars if x.dt > self.bi_list[-2].bars[-1].dt] \
                             + [x for x in bars_ubi if x.dt > self.bi_list[-1].bars[-1].dt]
                self.bi_list.pop(-1)
                self.bi_list.pop(-1)
            else:
                bars_ubi_a = last_bi.bars + [x for x in bars_ubi if x.dt > last_bi.bars[-1].dt]
                self.bi_list.pop(-1)
        elif last_bi.direction == Direction.Down and min_low_ubi < last_bi.low:
            if max_high_ubi > last_bi.high and len(self.bi_list) > 2:
                bars_ubi_a = self.bi_list[-2].bars \
                             + [x for x in self.bi_list[-1].bars if x.dt > self.bi_list[-2].bars[-1].dt] \
                             + [x for x in bars_ubi if x.dt > self.bi_list[-1].bars[-1].dt]
                self.bi_list.pop(-1)
                self.bi_list.pop(-1)
            else:
                bars_ubi_a = last_bi.bars + [x for x in bars_ubi if x.dt > last_bi.bars[-1].dt]
                self.bi_list.pop(-1)
        else:
            bars_ubi_a = bars_ubi

        if len(bars_ubi_a) > 300:
            print("{} - {} 未完成笔延伸超长，延伸数量: {}".format(self.symbol, self.freq, len(bars_ubi_a)))
        bi, bars_ubi_ = check_bi(bars_ubi_a)
        self.bars_ubi = bars_ubi_
        if isinstance(bi, BI):
            self.bi_list.append(bi)

    def get_signals(self):
        s = OrderedDict({"symbol": self.symbol, "dt": self.bars_raw[-1].dt, "close": self.bars_raw[-1].close})
        # 倒1，倒数第1笔的缩写，表示第N笔
        # 倒2，倒数第2笔的缩写，表示第N-1笔
        # 倒3，倒数第3笔的缩写，表示第N-2笔
        # 以此类推
        s.update({
            "未完成笔长度": len(self.bars_ubi),
            "三K形态": Signals.Other.value,

            "倒1方向": Signals.Other.value,
            "倒1长度": 0,
            "倒1价差力度": 0,
            "倒1涨跌幅": 0,
            "倒1拟合优度": 0,
            "倒1分型数量": 0,

            "倒2方向": Signals.Other.value,
            "倒2长度": 0,
            "倒2价差力度": 0,
            "倒2涨跌幅": 0,
            "倒2拟合优度": 0,
            "倒2分型数量": 0,

            "倒3方向": Signals.Other.value,
            "倒3长度": 0,
            "倒3价差力度": 0,
            "倒3涨跌幅": 0,
            "倒3拟合优度": 0,
            "倒3分型数量": 0,

            "倒4方向": Signals.Other.value,
            "倒4长度": 0,
            "倒4价差力度": 0,
            "倒4涨跌幅": 0,
            "倒4拟合优度": 0,
            "倒4分型数量": 0,

            "倒5方向": Signals.Other.value,
            "倒5长度": 0,
            "倒5价差力度": 0,
            "倒5涨跌幅": 0,
            "倒5拟合优度": 0,
            "倒5分型数量": 0,

            "倒1表里关系": Signals.Other.value,
            "倒1多头区间": False,
            "倒1空头区间": False,

            "倒1三笔": Signals.Other.value,
            "倒2三笔": Signals.Other.value,
            "倒3三笔": Signals.Other.value,
            "倒4三笔": Signals.Other.value,
            "倒5三笔": Signals.Other.value,

            "倒1形态": Signals.Other.value,
            "倒2形态": Signals.Other.value,
            "倒3形态": Signals.Other.value,
            "倒4形态": Signals.Other.value,
            "倒5形态": Signals.Other.value,
            "倒6形态": Signals.Other.value,
            "倒7形态": Signals.Other.value,
        })

        if len(self.bars_ubi) >= 3:
            tri = self.bars_ubi[-3:]
            if tri[0].high > tri[1].high < tri[2].high:
                s["三K形态"] = Signals.TK1.value
            elif tri[0].high < tri[1].high < tri[2].high:
                s["三K形态"] = Signals.TK2.value
            elif tri[0].high < tri[1].high > tri[2].high:
                s["三K形态"] = Signals.TK3.value
            elif tri[0].high > tri[1].high > tri[2].high:
                s["三K形态"] = Signals.TK4.value

        # 表里关系的定义参考：http://blog.sina.com.cn/s/blog_486e105c01007wc1.html
        min_ubi = min([x.low for x in self.bars_ubi])
        max_ubi = max([x.high for x in self.bars_ubi])

        if self.bi_list:
            last_bi = self.bi_list[-1]
            if last_bi.direction == Direction.Down:
                if min_ubi < last_bi.low:
                    s['倒1表里关系'] = Signals.BD1.value
                else:
                    s['倒1表里关系'] = Signals.BD0.value
            if last_bi.direction == Direction.Up:
                if max_ubi > last_bi.high:
                    s['倒1表里关系'] = Signals.BU1.value
                else:
                    s['倒1表里关系'] = Signals.BU0.value

        if s['倒1表里关系'] in [Signals.BU0.value, Signals.BD0.value]:
            bis = self.bi_list
        else:
            bis = self.bi_list[:-1]
            if self.bi_list:
                s['未完成笔长度'] = len(self.bars_ubi) + self.bi_list[-1].length - 3

        if not bis:
            return s

        for i in range(1, min(6, len(bis))):
            s['倒{}方向'.format(i)] = bis[-i].direction.value
            s['倒{}长度'.format(i)] = bis[-i].length
            s['倒{}价差力度'.format(i)] = bis[-i].power
            s['倒{}涨跌幅'.format(i)] = bis[-i].change
            s['倒{}拟合优度'.format(i)] = bis[-i].rsq
            s['倒{}分型数量'.format(i)] = len(bis[-i].fxs)

        # 笔的交易区间定义为 fx_b 的高低点
        s['倒1多头区间'] = bis[-1].direction == Direction.Down and bis[-1].fx_b.low < self.bars_ubi[-1].low < bis[-1].fx_b.high
        s['倒1空头区间'] = bis[-1].direction == Direction.Up and bis[-1].fx_b.high > self.bars_ubi[-1].high > bis[-1].fx_b.low

        s['倒1三笔'] = check_three_fd(bis[-3:])
        s['倒2三笔'] = check_three_fd(bis[-4:-1])
        s['倒3三笔'] = check_three_fd(bis[-5:-2])
        s['倒4三笔'] = check_three_fd(bis[-6:-3])
        s['倒5三笔'] = check_three_fd(bis[-7:-4])

        d1 = [check_five_fd(bis[-5:]), check_seven_fd(bis[-7:]), check_nine_fd(bis[-9:]), check_eleven_fd(bis[-11:])]
        for v in d1:
            if v != Signals.Other.value:
                s['倒1形态'] = v

        for i in range(2, 8):
            last_i = 1 - i
            v_seq = [check_five_fd(bis[last_i-5:last_i]), check_seven_fd(bis[last_i-7:last_i]),
                     check_nine_fd(bis[last_i-9:last_i]), check_eleven_fd(bis[last_i-11:last_i])]
            for v in v_seq:
                if v != Signals.Other.value:
                    s[f'倒{i}形态'] = v
        return s

    def update(self, bar: RawBar):
        """更新分析结果

        :param bar: 单根K线对象
        """
        # 更新K线序列
        if not self.bars_raw or bar.dt != self.bars_raw[-1].dt:
            self.bars_raw.append(bar)
            last_bars = [bar]
        else:
            self.bars_raw[-1] = bar
            last_bars = self.bars_ubi[-1].elements
            last_bars[-1] = bar
            self.bars_ubi.pop(-1)

        # 去除包含关系
        bars_ubi = self.bars_ubi
        for bar in last_bars:
            if len(bars_ubi) < 2:
                bars_ubi.append(NewBar(symbol=bar.symbol, dt=bar.dt, open=bar.open, close=bar.close,
                                       high=bar.high, low=bar.low, vol=bar.vol, elements=[bar]))
            else:
                k1, k2 = bars_ubi[-2:]
                has_include, k3 = remove_include(k1, k2, bar)
                if has_include:
                    bars_ubi[-1] = k3
                else:
                    bars_ubi.append(k3)
        self.bars_ubi = bars_ubi

        # 更新笔
        self.__update_bi()
        self.bi_list = self.bi_list[-self.max_bi_count:]
        if self.bi_list:
            sdt = self.bi_list[0].fx_a.elements[0].dt
            s_index = 0
            for i, bar in enumerate(self.bars_raw):
                if bar.dt >= sdt:
                    s_index = i
                    break
            self.bars_raw = self.bars_raw[s_index:]
        self.signals = self.get_signals()

    def to_echarts(self, width: str = "1400px", height: str = '580px'):
        kline = [x.__dict__ for x in self.bars_raw]
        if len(self.bi_list) > 0:
            bi = [{'dt': x.fx_a.dt, "bi": x.fx_a.fx} for x in self.bi_list] + \
                 [{'dt': self.bi_list[-1].fx_b.dt, "bi": self.bi_list[-1].fx_b.fx}]
        else:
            bi = None
        chart = kline_pro(kline, bi=bi, width=width, height=height, title="{}-{}".format(self.symbol, self.freq))
        return chart

    def open_in_browser(self, width: str = "1400px", height: str = '580px'):
        """直接在浏览器中打开分析结果

        :param width: 图表宽度
        :param height: 图表高度
        :return:
        """
        home_path = os.path.expanduser("~")
        file_html = os.path.join(home_path, "temp_czsc.html")
        chart = self.to_echarts(width, height)
        chart.render(file_html)
        webbrowser.open(file_html)




import os
import webbrowser
from typing import Set, List, Tuple
from collections import OrderedDict
from pyecharts.charts import Tab
from pyecharts.components import Table
from pyecharts.options import ComponentTitleOpts

from ..analyze import CZSC, get_sub_bis
from ..utils.kline_generator import KlineGeneratorBy1Min, KlineGeneratorByTick
from ..objects import RawBar
from ..enum import Factors
from .bi123 import future_bi123_f15, share_bi123_f15
from .bi_end import future_bi_end_f30, share_bi_end_f60
from .third_buy import future_third_buy_f5, share_third_buy_f5, share_third_buy_f15


def share_base(s):
    """股票基准"""
    return Factors.L1A0.value

def future_base(s):
    """期货基准"""
    return Factors.L1A0.value


factors_func = {
    share_base, future_base,

    future_bi123_f15, share_bi123_f15,

    future_bi_end_f30, share_bi_end_f60,

    future_third_buy_f5, share_third_buy_f5, share_third_buy_f15
}

factors_all = {f.__doc__: f for f in factors_func}

def aware_level_pairs(c6: CZSC, c5: CZSC, c4: CZSC, c3: CZSC, c2: CZSC, c1: CZSC):
    """感知级别配对情况

    :param c6: 日线 CZSC 对象
    :param c5: 60分钟 CZSC 对象
    :param c4: 30分钟 CZSC 对象
    :param c3: 15分钟 CZSC 对象
    :param c2: 5分钟 CZSC 对象
    :param c1: 1分钟 CZSC 对象
    :return:
    """
    # 找出日线笔对应的次级别
    bis_c6_c4 = get_sub_bis(c4.bi_list[-15:], c6.bi_list[-1])
    if 11 >= len(bis_c6_c4) >= 3:
        c6_sub = c4
    elif len(bis_c6_c4) > 11:
        c6_sub = c5
    else:
        c6_sub = c3

    # 找出60分钟笔对应的次级别
    bis_c5_c3 = get_sub_bis(c3.bi_list[-15:], c5.bi_list[-1])
    if 11 >= len(bis_c5_c3) >= 3:
        c5_sub = c3
    elif len(bis_c5_c3) > 11:
        c5_sub = c4
    else:
        c5_sub = c2

    # 找出30分钟笔对应的次级别
    bis_c4_c2 = get_sub_bis(c2.bi_list[-15:], c4.bi_list[-1])
    if 11 >= len(bis_c4_c2) >= 3:
        c4_sub = c2
    elif len(bis_c4_c2) > 11:
        c4_sub = c3
    else:
        c4_sub = c1

    # 找出15分钟笔对应的次级别
    bis_c3_c2 = get_sub_bis(c2.bi_list[-15:], c3.bi_list[-1])
    if 11 >= len(bis_c3_c2) >= 3:
        c3_sub = c2
    else:
        c3_sub = c1

    pairs = {
        c6.freq: c6_sub.freq,
        c5.freq: c5_sub.freq,
        c4.freq: c4_sub.freq,
        c3.freq: c3_sub.freq,
        c2.freq: c1.freq,
        c1.freq: c1.freq,
    }
    return pairs


class CzscFactors:
    """缠中说禅技术分析理论之多级别联立因子"""
    def __init__(self, kg: [KlineGeneratorByTick, KlineGeneratorBy1Min], factors: [Set, List, Tuple] = None):
        """

        :param kg: 基于tick或1分钟的K线合成器
        """
        self.name = "CzscFactors"
        self.kg = kg
        self.freqs = kg.freqs
        self.factors = factors

        klines = self.kg.get_klines({k: 3000 for k in self.freqs})
        self.kas = {k: CZSC(klines[k], freq=k, max_bi_count=30) for k in klines.keys()}
        self.symbol = self.kas["1分钟"].symbol
        self.end_dt = self.kas["1分钟"].bars_raw[-1].dt
        self.latest_price = self.kas["1分钟"].bars_raw[-1].close
        self.s = self._cal_signals()
        self.calculate_factors()
        self.cache = OrderedDict()

    def __repr__(self):
        return "<{} for {}>".format(self.name, self.symbol)

    def take_snapshot(self, file_html=None, width="1400px", height="580px"):
        """获取快照

        :param file_html: str
            交易快照保存的 html 文件名
        :param width: str
            图表宽度
        :param height: str
            图表高度
        :return:
        """
        tab = Tab(page_title="{}@{}".format(self.symbol, self.end_dt.strftime("%Y-%m-%d %H:%M")))
        for freq in self.freqs:
            chart = self.kas[freq].to_echarts(width, height)
            tab.add(chart, freq)

        for freq in self.freqs:
            t1 = Table()
            t1.add(["名称", "数据"], [[k, v] for k, v in self.s.items() if k.startswith("{}_".format(freq))])
            t1.set_global_opts(title_opts=ComponentTitleOpts(title="缠中说禅信号表", subtitle=""))
            tab.add(t1, "{}信号表".format(freq))

        t2 = Table()
        ths_ = [["同花顺F10",  "http://basic.10jqka.com.cn/{}".format(self.symbol[:6])]]
        t2.add(["名称", "数据"], [[k, v] for k, v in self.s.items() if "_" not in k] + ths_)
        t2.set_global_opts(title_opts=ComponentTitleOpts(title="缠中说禅因子表", subtitle=""))
        tab.add(t2, "因子表")

        t3 = Table()
        t3.add(["本级别", "次级别"], [[k, v] for k, v in eval(self.s['级别映射']).items()])
        t3.set_global_opts(title_opts=ComponentTitleOpts(title="缠中说禅级别映射表", subtitle=""))
        tab.add(t3, "级别映射表")

        if file_html:
            tab.render(file_html)
        else:
            return tab

    def open_in_browser(self, width="1400px", height="580px"):
        """直接在浏览器中打开分析结果"""
        home_path = os.path.expanduser("~")
        file_html = os.path.join(home_path, "temp_czsc_factors.html")
        self.take_snapshot(file_html, width, height)
        webbrowser.open(file_html)

    def _cal_signals(self):
        """计算信号"""
        s = OrderedDict()
        for freq, ks in self.kas.items():
            s.update({"{}_{}".format(ks.freq, k) if k not in ['symbol', 'dt', 'close'] else k: v
                      for k, v in ks.signals.items()})

        s.update(self.kas['1分钟'].bars_raw[-1].__dict__)
        return s

    def calculate_factors(self):
        """在这里定义因子计算的顺序，同时也可以根据需要，仅计算自己感兴趣的因子"""
        c1: CZSC = self.kas['1分钟']
        c2: CZSC = self.kas['5分钟']
        c3: CZSC = self.kas['15分钟']
        c4: CZSC = self.kas['30分钟']
        c5: CZSC = self.kas['60分钟']
        c6: CZSC = self.kas['日线']
        level_pairs = aware_level_pairs(c6, c5, c4, c3, c2, c1)
        self.s.update({"级别列表": self.kg.freqs, "级别映射": str(level_pairs)})
        if self.factors:
            for func in self.factors:
                self.s.update({func.__doc__: func(self.s)})

    def update_factors(self, k: RawBar):
        """更新多级别联立因子"""
        self.kg.update(k)
        klines_one = self.kg.get_klines({freq: 1 for freq in self.freqs})

        for freq, klines_ in klines_one.items():
            self.kas[freq].update(klines_[-1])

        self.symbol = self.kas["1分钟"].symbol
        self.end_dt = self.kas["1分钟"].bars_raw[-1].dt
        self.latest_price = self.kas["1分钟"].bars_raw[-1].close
        self.s = self._cal_signals()
        self.calculate_factors()



# coding: utf-8
import os
import pandas as pd
from czsc.utils.kline_generator import KlineGeneratorBy1Min as KlineGenerator
from czsc.objects import RawBar


cur_path = os.path.split(os.path.realpath(__file__))[0]
# cur_path = r"D:\git_repo\offline\dealer\test\czsc"
file_kline = os.path.join(cur_path, "data/000001.XSHG_1MIN.csv")
kline = pd.read_csv(file_kline, encoding="utf-8")
kline.loc[:, "dt"] = pd.to_datetime(kline.dt)
kline = [RawBar(symbol=row['symbol'], open=row['open'], dt=row['dt'],
                close=row['close'], high=row['high'], low=row['low'], vol=row['vol'])
         for _, row in kline.iterrows()]


def test_kline_generator():
    # 验证指定级别
    kg = KlineGenerator(freqs=['周线', '日线', '30分钟', '5分钟', '1分钟'])
    for row in kline:
        kg.update(row)

    assert not kg.m60 and not kg.m15

    kg = KlineGenerator(max_count=2000, freqs=['周线', '日线', '60分钟', '30分钟', '15分钟', '5分钟', '1分钟'])
    for row in kline:
        kg.update(row)

    assert kg.m60 and kg.m15

    # 验证K线获取
    klines = kg.get_klines({'1分钟': 100, '5分钟': 100})
    for k, v in klines.items():
        assert len(v) == 100

    # 数量验证
    assert len(kg.m1) == 2000
    assert len(kg.m5) == len(kline) // 5
    assert len(kg.m15) == len(kline) // 15
    assert len(kg.m30) == len(kline) // 30
    assert len(kg.m60) == len(kline) // 60
    assert len(kg.D) == len(kline) // 240

    # 验证周线
    assert kg.W[-1].open == 3379.39
    assert kg.W[-1].close == 3361.3
    assert kg.W[-1].high == 3458.79
    assert kg.W[-1].low == 3345.75

    # 测试实盘连续输入
    for _ in range(5):
        kg.update(kline[-1])
        assert len(kg.m1) == 2000
        assert len(kg.m5) == len(kline) // 5
        assert len(kg.m30) == len(kline) // 30
        assert len(kg.D) == len(kline) // 240
        assert kg.W[-1].open == 3379.39
        assert kg.W[-1].close == 3361.3
        assert kg.W[-1].high == 3458.79
        assert kg.W[-1].low == 3345.75




# coding: utf-8
from dataclasses import dataclass
from datetime import datetime
from typing import List
from .enum import Mark, Direction

@dataclass
class Tick:
    symbol: str
    name: str = ""
    price: float = 0
    vol: float = 0

@dataclass
class RawBar:
    """原始K线元素"""
    symbol: str
    dt: datetime = None
    # freq: str = None
    open: [float, int] = None
    close: [float, int] = None
    high: [float, int] = None
    low: [float, int] = None
    vol: [float, int] = None

@dataclass
class NewBar:
    """去除包含关系的K线元素"""
    symbol: str
    dt: datetime = None
    # freq: str = None
    open: [float, int] = None
    close: [float, int] = None
    high: [float, int] = None
    low: [float, int] = None
    vol: [float, int] = None
    elements: List[RawBar] = None   # 存入具有包含关系的原始K线

@dataclass
class FX:
    symbol: str
    dt: datetime = None
    mark: Mark = None
    high: float = None
    low: float = None
    fx: float = None
    power: str = None
    elements: List[NewBar] = None

@dataclass
class BI:
    symbol: str
    fx_a: FX = None    # 笔开始的分型
    fx_b: FX = None    # 笔结束的分型
    fxs: List[FX] = None    # 笔内部的分型列表
    direction: Direction = None
    high: float = None
    low: float = None
    power: float = None
    bars: List[NewBar] = None
    rsq: float = None
    change: float = None
    length: float = None

@dataclass
class FakeBI:
    """虚拟笔：主要为笔的内部分析提供便利"""
    symbol: str
    direction: Direction = None
    high: float = None
    low: float = None
    power: float = None





from typing import List
from .factors import CzscFactors, factors_all
from ..enum import Factors
from .trader import CzscTrader


def get_trade_factors(name: str,
                      mp: float,
                      allow_zero: bool,
                      long_open_values: List,
                      long_close_values: List,
                      short_open_values: List = None,
                      short_close_values: List = None) -> dict:
    """获取指定 name 下的交易因子

    :param allow_zero: 是否使用基础型
    :param name: 因子系统的名称
    :param mp: 单个标的最大允许持仓，小于0表示仓位百分比，大于0表示手数
    :param long_open_values: 开多因子值
    :param long_close_values: 平多因子值
    :param short_open_values: 开空因子值
    :param short_close_values: 平空因子值
    :return: 因子交易系统

    example:
    ===================
    >>> factors = get_trade_factors(name="日线笔结束", long_open_values=['BDE'], long_close_values=['BUE'])
    """
    if not short_close_values:
        short_close_values = []

    if not short_open_values:
        short_open_values = []

    def __is_match(v, x):
        if allow_zero:
            if v in x.name:
                return 1
            else:
                return 0
        else:
            if v in x.name and "0" not in x.name:
                return 1
            else:
                return 0

    long_open_factors = ["{}@{}".format(name, x.value) for x in Factors.__members__.values()
                         if sum([__is_match(v, x) for v in long_open_values]) > 0]

    long_close_factors = ["{}@{}".format(name, x.value) for x in Factors.__members__.values()
                          if sum([__is_match(v, x) for v in long_close_values]) > 0]

    short_open_factors = ["{}@{}".format(name, x.value) for x in Factors.__members__.values()
                          if sum([__is_match(v, x) for v in short_open_values]) > 0]

    short_close_factors = ["{}@{}".format(name, x.value) for x in Factors.__members__.values()
                           if sum([__is_match(v, x) for v in short_close_values]) > 0]

    factors_ = {
        "name": name,
        "version": factors_all[name].__name__,
        "mp": mp,
        "long_open_factors": long_open_factors,
        "long_close_factors": long_close_factors,
        "short_open_factors": short_open_factors,
        "short_close_factors": short_close_factors,
    }
    return factors_





# coding: utf-8

from .analyze import CZSC
from .factors.factors import CzscFactors
from .utils.ta import SMA, EMA, MACD, KDJ

__version__ = "0.7.1"
__author__ = "zengbin93"
__email__ = "zeng_bin8888@163.com"




# coding: utf-8
import sys
import warnings

sys.path.insert(0, '.')
sys.path.insert(0, '..')
import os
import pandas as pd
import numpy as np
import czsc

warnings.warn("czsc version is {}".format(czsc.__version__))

cur_path = os.path.split(os.path.realpath(__file__))[0]
file_kline = os.path.join(cur_path, "data/000001.SH_D.csv")
kline = pd.read_csv(file_kline, encoding="utf-8")
kline.loc[:, "dt"] = pd.to_datetime(kline.dt)
bars = kline.to_dict("records")
close = np.array([x['close'] for x in bars], dtype=np.double)


def test_sma():
    ma5 = czsc.SMA(close, 5)
    assert len(ma5) == len(close)
    assert round(ma5[-1], 2) == 3362.53
    assert round(ma5[-2], 2) == 3410.62


def test_macd():
    diff, dea, macd = czsc.MACD(close)

    assert len(diff) == len(dea) == len(macd) == len(close)
    assert round(macd[-1], 2) == 13.35
    assert round(macd[-5], 2) == 88.0

    assert round(diff[-1], 2) == 117.3
    assert round(diff[-5], 2) == 127.51

    assert round(dea[-1], 2) == 110.62
    assert round(dea[-5], 2) == 83.51


def test_jdk():
    high = np.array([x['high'] for x in bars], dtype=np.double)
    low = np.array([x['low'] for x in bars], dtype=np.double)
    k, d, j = czsc.KDJ(close, high, low)

    assert round(k[-1], 2) == 59.94
    assert round(d[-1], 2) == 80.47
    assert round(j[-1], 2) == 18.87



# coding: utf-8
import warnings
from typing import List, Dict, OrderedDict
from ..enum import Signals, Factors, Freq

# ======================================================================================================================
def future_bi123_f15_base(s: [Dict, OrderedDict]):
    """期货15分钟123C"""
    v = Factors.Other.value
    freq = Freq.F30.value

    for f_ in [freq]:
        if f_ not in s['级别列表']:
            warnings.warn(f"{f_} not in {s['级别列表']}，默认返回 Other")
            return v

    # 开多仓因子
    # ------------------------------------------------------------------------------------------------------------------
    # 倒1一买
    d1_b1 = [Signals.LB0.value, Signals.LC0.value, Signals.LC1.value, Signals.LD0.value,
             Signals.LE0.value, Signals.LE1.value, Signals.LF0.value]
    # d1_b1 = [Signals.LB0.value, Signals.LC0.value, Signals.LD0.value,
    #          Signals.LE0.value, Signals.LF0.value]
    if s[f'{freq}_倒1形态'] in d1_b1 and s[f'{freq}_倒1多头区间']:
        v = Factors.L1A0.value

    # 平多仓因子
    # ------------------------------------------------------------------------------------------------------------------
    # 倒1一卖
    # d1_s1 = [
    #     Signals.SA0.value, Signals.SB0.value, Signals.SC0.value, Signals.SC1.value,
    #     Signals.SD0.value, Signals.SE0.value, Signals.SE1.value, Signals.SF0.value
    # ]
    # d1_s1 = [
    #     Signals.SA0.value, Signals.SB0.value, Signals.SC0.value,
    #     Signals.SD0.value, Signals.SE0.value, Signals.SF0.value
    # ]
    # if s[f'{freq}_倒1形态'] in d1_s1 and s[f'{freq}_倒1空头区间']:
    #     v = Factors.S1A0.value
    return v

def future_bi123_f15_v1(s: [Dict, OrderedDict]):
    """期货15分钟123"""
    v = Factors.Other.value
    freq = Freq.F15.value

    for f_ in [freq]:
        if f_ not in s['级别列表']:
            warnings.warn(f"{f_} not in {s['级别列表']}，默认返回 Other")
            return v

    # 开多仓因子
    # --------------------------------------------------------------------------------------------------------------
    d1_b1 = [Signals.LB0.value, Signals.LD0.value, Signals.LE0.value, Signals.LF0.value]
    if s[f'{freq}_倒1形态'] in d1_b1 and s[f'{freq}_倒1多头区间']:
        v = Factors.L1A0.value

    if (s[f'{freq}_倒5形态'] in d1_b1 or s[f'{freq}_倒7形态'] in d1_b1) \
            and s[f'{freq}_倒1形态'] in [Signals.LG0.value] and s[f'{freq}_倒1多头区间']:
        v = Factors.L2A0.value

    # 平多仓因子
    # --------------------------------------------------------------------------------------------------------------
    d1_s1 = [
        Signals.SA0.value, Signals.SB0.value, Signals.SC0.value,
        Signals.SD0.value, Signals.SE0.value, Signals.SF0.value
    ]
    if s[f'{freq}_倒1形态'] in d1_s1 and s[f'{freq}_倒1空头区间']:
        v = Factors.S1A0.value
    return v

def future_bi123_f15_v2(s: [Dict, OrderedDict]):
    """期货15分钟123"""
    v = Factors.Other.value
    freq = Freq.F15.value

    for f_ in [freq]:
        if f_ not in s['级别列表']:
            warnings.warn(f"{f_} not in {s['级别列表']}，默认返回 Other")
            return v

    # 开多仓因子
    # --------------------------------------------------------------------------------------------------------------
    d1_b1 = [Signals.LB0.value, Signals.LD0.value, Signals.LE0.value, Signals.LF0.value]
    if s[f'{freq}_倒1形态'] in d1_b1 and s[f'{freq}_倒1多头区间']:
        v = Factors.L1A0.value

    if (s[f'{freq}_倒5形态'] in d1_b1 or s[f'{freq}_倒7形态'] in d1_b1) \
            and s[f'{freq}_倒1形态'] in [Signals.LG0.value, Signals.LH0.value] and s[f'{freq}_倒1多头区间']:
        v = Factors.L2A0.value

    if (s[f'{freq}_倒5形态'] in d1_b1 or s[f'{freq}_倒7形态'] in d1_b1) \
            and s[f'{freq}_倒1形态'] in [Signals.LI0.value] and s[f'{freq}_倒1多头区间']:
        v = Factors.L2A0.value

    # 平多仓因子
    # --------------------------------------------------------------------------------------------------------------
    d1_s1 = [
        Signals.SA0.value, Signals.SB0.value, Signals.SC0.value,
        Signals.SD0.value, Signals.SE0.value, Signals.SF0.value
    ]
    if s[f'{freq}_倒1形态'] in d1_s1 and s[f'{freq}_倒1空头区间']:
        v = Factors.S1A0.value
    return v

def future_bi123_f15_v3(s: [Dict, OrderedDict]):
    """期货15分钟123"""
    v = Factors.Other.value
    freq = Freq.F15.value

    for f_ in [freq]:
        if f_ not in s['级别列表']:
            warnings.warn(f"{f_} not in {s['级别列表']}，默认返回 Other")
            return v
    # 平多仓因子
    # --------------------------------------------------------------------------------------------------------------
    d1_s1 = [
        Signals.SA0.value, Signals.SB0.value, Signals.SC0.value,
        Signals.SD0.value, Signals.SE0.value, Signals.SF0.value
    ]
    for freq in [Freq.F5.value, Freq.F1.value]:
        if s[f'{freq}_倒1形态'] in d1_s1 and s[f'{freq}_倒1空头区间']:
            v = Factors.S1A0.value

    # 开多仓因子
    # --------------------------------------------------------------------------------------------------------------
    d1_b1 = [Signals.LB0.value, Signals.LD0.value, Signals.LE0.value, Signals.LF0.value]
    if s[f'{freq}_倒1形态'] in d1_b1 and s[f'{freq}_倒1多头区间']:
        v = Factors.L1A0.value

    if (s[f'{freq}_倒5形态'] in d1_b1 or s[f'{freq}_倒7形态'] in d1_b1) \
            and s[f'{freq}_倒1形态'] in [Signals.LG0.value, Signals.LH0.value] \
            and s[f'{freq}_倒1多头区间']:
        v = Factors.L2A0.value

    if (s[f'{freq}_倒5形态'] in d1_b1 or s[f'{freq}_倒7形态'] in d1_b1) \
            and s[f'{freq}_倒1形态'] in [Signals.LI0.value] and s[f'{freq}_倒1多头区间']:
        v = Factors.L2A0.value
    return v


future_bi123_f15 = future_bi123_f15_base
# ======================================================================================================================
def share_bi123_f15_base(s: [Dict, OrderedDict]):
    """股票15分钟123"""
    v = Factors.Other.value
    freq = Freq.F15.value

    for f_ in [freq]:
        if f_ not in s['级别列表']:
            warnings.warn(f"{f_} not in {s['级别列表']}，默认返回 Other")
            return v

    # 开多仓因子
    # --------------------------------------------------------------------------------------------------------------
    # 倒1一买
    d1_b1 = [Signals.LB0.value, Signals.LC0.value, Signals.LE0.value, Signals.LF0.value]
    if s[f'{freq}_倒1形态'] in d1_b1 and s[f'{freq}_倒1多头区间']:
        v = Factors.L1A0.value

    # 平多仓因子
    # --------------------------------------------------------------------------------------------------------------
    # 倒1一卖
    d1_s1 = [
        Signals.SA0.value, Signals.SB0.value, Signals.SC0.value,
        Signals.SD0.value, Signals.SE0.value, Signals.SF0.value
    ]
    if s[f'{freq}_倒1形态'] in d1_s1 and s[f'{freq}_倒1空头区间']:
        v = Factors.S1A0.value
    return v

def share_bi123_f15_v1(s: [Dict, OrderedDict]):
    """股票15分钟123"""
    v = Factors.Other.value
    freq = Freq.F15.value

    for f_ in [freq]:
        if f_ not in s['级别列表']:
            warnings.warn(f"{f_} not in {s['级别列表']}，默认返回 Other")
            return v
    # 平多仓因子
    # --------------------------------------------------------------------------------------------------------------
    d1_s1 = [
        Signals.SA0.value, Signals.SB0.value, Signals.SC0.value,
        Signals.SD0.value, Signals.SE0.value, Signals.SF0.value
    ]
    for freq in [Freq.F5.value, Freq.F1.value]:
        if s[f'{freq}_倒1形态'] in d1_s1 and s[f'{freq}_倒1空头区间']:
            v = Factors.S1A0.value

    # 开多仓因子
    # --------------------------------------------------------------------------------------------------------------
    d1_b1 = [Signals.LB0.value, Signals.LD0.value, Signals.LE0.value, Signals.LF0.value]
    if s[f'{freq}_倒1形态'] in d1_b1 and s[f'{freq}_倒1多头区间']:
        v = Factors.L1A0.value

    if (s[f'{freq}_倒5形态'] in d1_b1 or s[f'{freq}_倒7形态'] in d1_b1) \
            and s[f'{freq}_倒1形态'] in [Signals.LG0.value, Signals.LH0.value] \
            and s[f'{freq}_倒1多头区间']:
        v = Factors.L2A0.value

    if (s[f'{freq}_倒5形态'] in d1_b1 or s[f'{freq}_倒7形态'] in d1_b1) \
            and s[f'{freq}_倒1形态'] in [Signals.LI0.value] and s[f'{freq}_倒1多头区间']:
        v = Factors.L2A0.value
    return v


share_bi123_f15 = share_bi123_f15_v1
# ======================================================================================================================



# coding: utf-8
import warnings
from typing import List, Dict, OrderedDict
from ..enum import Signals, Factors, Freq

def third_buy_base(s: [Dict, OrderedDict], freqs: List[Freq], sub_freqs: List[Freq]):
    """

    :param s: 信号表
    :param freqs: 本级别名称
    :param sub_freqs: 次级别名称
    :return:
    """
    v = Factors.Other.value

    for freq in freqs + sub_freqs:
        if freq not in s['级别列表']:
            warnings.warn(f"{freq} not in {s['级别列表']}，默认返回 Other")
            return v

    for freq in freqs:
        if s[f'{freq}_倒1形态'] in [Signals.LI0.value]:
            v = Factors.L3A0.value

    for freq in freqs:
        xt = [
            Signals.SA0.value, Signals.SB0.value, Signals.SC0.value,
            Signals.SD0.value, Signals.SE0.value, Signals.SF0.value
        ]
        if s[f'{freq}_倒1形态'] in xt:
            v = Factors.S1A0.value

    return v

# ======================================================================================================================
def future_third_buy_f5_base(s):
    """期货5分钟三买"""
    return third_buy_base(s, [Freq.F5.value], [Freq.F1.value])


future_third_buy_f5 = future_third_buy_f5_base
# ======================================================================================================================
def share_third_buy_f5_base(s):
    """股票5分钟三买"""
    return third_buy_base(s, [Freq.F5.value], [Freq.F1.value])

def share_third_buy_f15_base(s):
    """股票15分钟三买"""
    return third_buy_base(s, [Freq.F15.value], [Freq.F1.value])


share_third_buy_f5 = share_third_buy_f5_base
share_third_buy_f15 = share_third_buy_f15_base
# ======================================================================================================================

